import type { Server as SocketIoServer } from "socket.io";
import { Express } from "express";
import { Contract, HttpProvider, TransactionReceiptBase, WebSocketProvider } from "web3";
import { ConsensusAlgorithmFamily, IPluginLedgerConnector, IWebServiceEndpoint, IPluginWebService, ICactusPlugin, ICactusPluginOptions } from "@hyperledger/cactus-core-api";
import { PluginRegistry } from "@hyperledger/cactus-core";
import { LogLevelDesc } from "@hyperledger/cactus-common";
import { DeployContractV1Request, InvokeContractV1Request, InvokeContractV1Response, RunTransactionRequest, RunTransactionResponse, InvokeRawWeb3EthMethodV1Request, InvokeRawWeb3EthContractV1Request } from "./generated/openapi/typescript-axios";
import { PrometheusExporter } from "./prometheus-exporter/prometheus-exporter";
export type HttpProviderOptions = ConstructorParameters<typeof HttpProvider>[1];
export type WsProviderSocketOptions = ConstructorParameters<typeof WebSocketProvider>[1];
export type WsProviderReconnectOptions = ConstructorParameters<typeof WebSocketProvider>[2];
export interface IPluginLedgerConnectorEthereumOptions extends ICactusPluginOptions {
    rpcApiHttpHost?: string;
    rpcApiHttpOptions?: HttpProviderOptions;
    rpcApiWsHost?: string;
    rpcApiWsSocketOptions?: WsProviderSocketOptions;
    rpcApiWsReconnectOptions?: WsProviderReconnectOptions;
    logLevel?: LogLevelDesc;
    prometheusExporter?: PrometheusExporter;
    pluginRegistry: PluginRegistry;
}
export declare class PluginLedgerConnectorEthereum implements IPluginLedgerConnector<DeployContractV1Request, RunTransactionResponse, RunTransactionRequest, RunTransactionResponse>, ICactusPlugin, IPluginWebService {
    readonly options: IPluginLedgerConnectorEthereumOptions;
    private readonly pluginRegistry;
    prometheusExporter: PrometheusExporter;
    private readonly instanceId;
    private readonly log;
    private readonly web3;
    private endpoints;
    static readonly CLASS_NAME = "PluginLedgerConnectorEthereum";
    private watchBlocksSubscriptions;
    get className(): string;
    private createWeb3WsProvider;
    private createWeb3Provider;
    constructor(options: IPluginLedgerConnectorEthereumOptions);
    getOpenApiSpec(): unknown;
    getPrometheusExporter(): PrometheusExporter;
    getPrometheusExporterMetrics(): Promise<string>;
    getInstanceId(): string;
    private removeWatchBlocksSubscriptionForSocket;
    private closeWeb3jsConnection;
    shutdown(): Promise<void>;
    onPluginInit(): Promise<unknown>;
    registerWebServices(app: Express, wsApi: SocketIoServer): Promise<IWebServiceEndpoint[]>;
    getOrCreateWebServices(): Promise<IWebServiceEndpoint[]>;
    getPackageName(): string;
    getConsensusAlgorithmFamily(): Promise<ConsensusAlgorithmFamily>;
    hasTransactionFinality(): Promise<boolean>;
    /**
     * Verifies that it is safe to call a specific method on an object.
     *
     * @param object Object instance to check whether it has a method with a specific name or not.
     * @param name The name of the method that will be checked if it's usable on `object` or not.
     * @returns Boolean `true` when it IS safe to call the method named `name` on the object.
     * @throws If the object instance is falsy or the method name is a blank string.
     */
    isSafeToCallObjectMethod(object: Record<string, unknown>, name: string): boolean;
    /**
     * Verifies that it is safe to call a specific method of a Web3 Contract.
     *
     * @param contract The Web3 Contract instance to check whether it has a method with a specific name or not.
     * @param name The name of the method that will be checked if it's usable on `contract` or not.
     * @returns Boolean `true` when it IS safe to call the method named `name` on the contract.
     * @throws If the contract instance is falsy or it's methods object is falsy. Also throws if the method name is a blank string.
     */
    isSafeToCallContractMethod(contract: InstanceType<typeof Contract>, name: string): Promise<boolean>;
    /**
     * Simple function for estimating `maxFeePerGas` to sent with transaction.
     * @warn It's not optimized for either speed or cost, consider using more complex solution on production!
     * @param priorityFee what priority tip you plan to include.
     * @returns estimated `maxFeePerGas` value.
     */
    estimateMaxFeePerGas(priorityFee?: number | string): Promise<string>;
    /**
     * Invoke contract method.
     *
     * @param req contract method and transaction definition
     * @returns transaction receipt
     */
    invokeContract(req: InvokeContractV1Request): Promise<InvokeContractV1Response>;
    /**
     *  Invoke contract method using contract instance stored in a keychain plugin.
     *
     * @param req contract method and transaction definition
     * @param contract contract keychain reference
     * @returns transaction receipt
     */
    private invokeContractFromKeychain;
    /**
     * Internal logic for invoking actual transaction given address and ABI.
     *
     * @param req contract method and transaction definition
     * @param contractAddress deployed contract address
     * @param contractJSON contract ABI and bytecode
     * @returns transaction receipt
     */
    private runContractInvoke;
    /**
     * Send ethereum transaction.
     *
     * @param req transaction definition.
     * @returns transaction receipt.
     */
    transact(req: RunTransactionRequest): Promise<RunTransactionResponse>;
    /**
     * Send already signed transaction.
     *
     * @param rawTransaction signed transaction payload.
     * @returns transaction receipt.
     */
    transactSigned(rawTransaction: string): Promise<RunTransactionResponse>;
    /**
     * Wait until receipt for transaction with specified hash is returned.
     * Throws if timeout expires.
     *
     * @param txHash sent transaction hash
     * @param timeoutMs timeout in milliseconds
     * @returns transaction receipt.
     */
    pollForTxReceipt(txHash: string, timeoutMs?: number): Promise<TransactionReceiptBase<string, string, string, unknown>>;
    /**
     * Transact with identity stored in Geth node.
     *
     * @param txIn transaction definition.
     * @returns transaction receipt.
     */
    private transactGethKeychain;
    /**
     * Transact with private key passed in argument.
     *
     * @param req transaction definition.
     * @returns transaction receipt.
     */
    private transactPrivateKey;
    /**
     * Transact with identity stored in cacti keychain.
     *
     * @param req transaction definition.
     * @returns transaction receipt.
     */
    private transactCactiKeychainRef;
    /**
     * Convert connector transaction config to web3js transaction object.
     *
     * @param txConfig connector transaction config
     * @returns web3js transaction
     */
    private getTransactionFromTxConfig;
    /**
     * Deploy contract to an Ethereum based ledger.
     *
     * @param req contract and transaction definition
     * @returns transaction receipt
     */
    deployContract(req: DeployContractV1Request): Promise<RunTransactionResponse>;
    /**
     * Internal function for creating and sending contract deployment transaction.
     *
     * @param config deployment configuration
     * @returns transaction receipt
     */
    private runContractDeployment;
    /**
     * Internal function for deploying contract that has it's JSON stored in a keychain plugin.
     *
     * @param req contract and transaction definition
     * @param contract contract on keychain definition
     * @returns transaction receipt
     */
    private deployContractFromKeychain;
    /**
     * Low level function to call any method from web3.eth.
     * Should be used only if given functionality is not already covered by another endpoint.
     *
     * @param args web3.eth method and arguments
     * @returns method response
     */
    invokeRawWeb3EthMethod(args: InvokeRawWeb3EthMethodV1Request): Promise<any>;
    /**
     * Low level function to invoke contract.
     * Should be used only if given functionality is not already covered by another endpoint.
     *
     * @param args web3 contract method and arguments
     * @returns method response
     */
    invokeRawWeb3EthContract(args: InvokeRawWeb3EthContractV1Request): Promise<any>;
}
