/**
 * Contains abstract base WatchBlocksV1Endpoint, it specific implementation (with WebSockets and HTTP polling) and some helper methods.
 * In the future we may consider dividing this into separate files, but for now (since codebase is small) it makes more sense to keep
 * everything in one place for simplicity.
 */
import Web3, { BlockHeaderOutput } from "web3";
import type { Socket as SocketIoSocket } from "socket.io";
import { Logger, LogLevelDesc } from "@hyperledger/cactus-common";
import { WatchBlocksV1Options, WatchBlocksV1Progress, WatchBlocksV1 } from "../generated/openapi/typescript-axios";
export interface IWatchBlocksV1EndpointConfiguration {
    logLevel?: LogLevelDesc;
    socket: SocketIoSocket;
    web3: InstanceType<typeof Web3>;
    options?: WatchBlocksV1Options;
}
/**
 * Factory method for creating either subscription based or HTTP-poll based `WatchBlocksV1Endpoint`
 * (depending on web3js instance capability)
 *
 * @param config `WatchBlocksV1Endpoint` configuration
 * @returns `WatchBlocksV1Endpoint`
 */
export declare function createWatchBlocksV1Endpoint(config: IWatchBlocksV1EndpointConfiguration): WatchBlocksV1Endpoint;
/**
 * Base abstract class to be extended by specific WatchBlocks strategies.
 * Contains common logic for all monitoring endpoints.
 */
export declare abstract class WatchBlocksV1Endpoint {
    readonly config: IWatchBlocksV1EndpointConfiguration;
    protected readonly log: Logger;
    protected readonly socket: SocketIoSocket<Record<WatchBlocksV1, (next: string) => void>, Record<WatchBlocksV1, (next: WatchBlocksV1Progress | string) => void>>;
    protected readonly web3: InstanceType<typeof Web3>;
    protected readonly isGetBlockData: boolean;
    protected lastSeenBlock: number;
    private _isSubscribed;
    /**
     * Note: should be overwritten by the strategy classes.
     */
    get className(): string;
    get isSubscribed(): boolean;
    set isSubscribed(value: boolean);
    constructor(config: IWatchBlocksV1EndpointConfiguration);
    /**
     * Get block with specific number from ethereum, parse it and return
     * `WatchBlocksV1Progress` with `blockData`.
     *
     * @param blockNumber
     * @returns `WatchBlocksV1Progress
     */
    protected getFullBlockProgress(blockNumber: number): Promise<WatchBlocksV1Progress>;
    /**
     * Parse block header data and return `WatchBlocksV1Progress` with `blockHeader`.
     * @param web3BlockData `BlockHeaderOutput`
     * @returns `WatchBlocksV1Progress`
     */
    protected headerDataToBlockProgress(web3BlockData: BlockHeaderOutput): Promise<WatchBlocksV1Progress>;
    /**
     * Push to the socketio client all missing blocks since `lastSeenBlock`, up to the current latest block.
     * Will not push if subscription was stopped in the meantime.
     * Can throw exceptions.
     */
    protected emitAllSinceLastSeenBlock(): Promise<void>;
    /**
     * Start monitoring new blocks from the ledger (either full blocks with transaction data or just the headers).
     * Will fetch and push missing blocks since `lastSeenBlock` provided.
     */
    abstract subscribe(): Promise<WatchBlocksV1Endpoint>;
    /**
     * Stop monitoring new blocks.
     */
    abstract unsubscribe(): Promise<void>;
}
/**
 * Implementation of `WatchBlocksV1Endpoint` using HTTP polling method.
 * Should be used if ethereum node (i.e. web3js connection) doesn't support
 * async subscriptions.
 */
export declare class WatchBlocksV1HttpPollEndpoint extends WatchBlocksV1Endpoint {
    readonly config: IWatchBlocksV1EndpointConfiguration;
    private monitoringInterval;
    private httpPollInterval;
    get className(): string;
    constructor(config: IWatchBlocksV1EndpointConfiguration);
    subscribe(): Promise<this>;
    unsubscribe(): Promise<void>;
}
/**
 * Implementation of `WatchBlocksV1Endpoint` using web3js subscription (WebSocket)
 * Should be used if ethereum node (i.e. web3js connection) supports async subscriptions.
 * Throws if wrong web3js provider was used.
 */
export declare class WatchBlocksV1SubscriptionEndpoint extends WatchBlocksV1Endpoint {
    readonly config: IWatchBlocksV1EndpointConfiguration;
    private newBlocksSubscription;
    get className(): string;
    constructor(config: IWatchBlocksV1EndpointConfiguration);
    subscribe(): Promise<this>;
    unsubscribe(): Promise<void>;
}
